import click
import sys
import logging
from rli.cli import CONTEXT_SETTINGS
from rli.github import RLIGithub
from rli.config import get_rli_config_or_exit
from rli.constants import ExitCode


@click.group(name="github", help="Contains all github commands for RLI.")
@click.pass_context
def cli(cts):
    # Click group for github commands
    pass


@cli.command(
    name="create-repo",
    context_settings=CONTEXT_SETTINGS,
    help="Creates a repo with the given information",
)
@click.option("--repo-name", default=None)
@click.option("--repo-description", default=None)
@click.option("--private", default="false")
@click.pass_context
def create_repo(ctx, repo_name, repo_description, private):
    repo = RLIGithub(get_rli_config_or_exit().github_config).create_repo(
        repo_name, repo_description, private
    )

    if repo:
        logging.info(f"Here is your new repo:\n{str(repo)}")
        sys.exit(ExitCode.OK)
    else:
        sys.exit(ExitCode.GITHUB_ERROR)



import json
import os
import sys
import logging
from rli.exceptions import InvalidRLIConfiguration, InvalidDeployConfiguration
from rli.constants import ExitCode


class DockerDeployConfig:
    def __init__(self, config):
        self.image = config.get("image") or None
        self.compose_file = config.get("compose_file") or None
        self.validate_config()

    def validate_config(self):
        message = ""

        if not self.image:
            message += "No docker image specified. "

        if message != "":
            raise InvalidDeployConfiguration(message)

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.image == other.image and self.compose_file == other.compose_file

        return False


class DeployConfig:
    def __init__(self):
        self.deploy_config_path = "deploy/deploy.json"
        with open(self.deploy_config_path, "r") as config:
            self.deploy_config = json.load(config)

        try:
            self.docker_deploy_config = DockerDeployConfig(self.deploy_config["docker"])
        except KeyError:
            self.docker_deploy_config = None

        self.secrets = self.deploy_config.get("secrets") or []

        self.validate_config()

    def validate_config(self):
        if not self.docker_deploy_config:
            raise InvalidDeployConfiguration("No configuration specified.")

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return (
                self.docker_deploy_config == other.docker_deploy_config
                and self.secrets == other.secrets
            )

        return False


class DockerConfig:
    def __init__(self, config):
        self.registry = config.get("registry") or None
        self.login = config.get("login") or None
        self.password = config.get("password") or None

        self.validate_config()

    def validate_config(self):
        message = ""

        if not self.registry:
            message += "Docker registry was not provided. "

        if not self.login:
            message += "Docker login was not provided. "

        if not self.password:
            message += "Docker password was not provided."

        if message != "":
            raise InvalidRLIConfiguration(message)

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return (
                self.registry == other.registry
                and self.login == other.login
                and self.password == other.password
            )

        return False


class GithubConfig:
    def __init__(self, config):
        self.organization = config.get("organization") or None
        self.login = config.get("login") or None
        self.password = config.get("password") or None

        self.validate_config()

    def validate_config(self):
        message = ""

        if not self.organization:
            message += "Github organization was not provided. "

        if not self.login:
            message += "Github login was not provided."

        if not self.password:
            self.password = ""

        if message != "":
            raise InvalidRLIConfiguration(message)

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return (
                self.organization == other.organization
                and self.login == other.login
                and self.password == other.password
            )

        return False


class RLIConfig:
    def __init__(self):
        self.home_dir = os.path.expanduser("~")

        self.rli_config = self.load_rli_config()
        self.rli_secrets = self.load_rli_secrets()

        self._github_config = None
        self._docker_config = None

    @property
    def github_config(self) -> GithubConfig:
        if self._github_config is None:
            github_config = self.rli_config.get("github") or None

            if not github_config:
                raise InvalidRLIConfiguration(
                    "Github configuration was not provided in ~/.rli/config.json."
                )

            self._github_config = GithubConfig(github_config)

        return self._github_config

    @property
    def docker_config(self) -> DockerConfig:
        if self._docker_config is None:
            docker_config = self.rli_config.get("docker") or None

            if not docker_config:
                raise InvalidRLIConfiguration(
                    "Docker configuration was not provided in ~/.rli/config.json."
                )

            self._docker_config = DockerConfig(docker_config)

        return self._docker_config

    def get_secret(self, key):
        value = ""
        try:
            value = self.rli_secrets[key]
        except KeyError:
            pass

        return value

    def load_rli_config(self):
        config = {}
        with open(f"{self.home_dir}/.rli/config.json", "r") as config:
            config = json.load(config)

        return config

    def load_rli_secrets(self):
        secrets = {}
        with open(f"{self.home_dir}/.rli/secrets.json", "r") as secrets:
            secrets = json.load(secrets)

        return secrets

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return (
                self.github_config == other.github_config
                and self.docker_config == other.docker_config
            )

        return False


def get_rli_config_or_exit() -> RLIConfig:
    config = None
    try:
        config = RLIConfig()
    # except InvalidRLIConfiguration as e:
    #     logging.exception("Your ~/.rli/config.json file is invalid.", e)
    #     sys.exit(ExitCode.INVALID_RLI_CONFIG)
    except FileNotFoundError:
        logging.exception("Could not find ~/.rli/config.json.")
        sys.exit(ExitCode.NO_RLI_CONFIG)

    return config


def get_deploy_config_or_exit() -> DeployConfig:
    config = None
    try:
        config = DeployConfig()
    except InvalidDeployConfiguration:
        logging.exception("Your config/config.json file is invalid.")
        sys.exit(ExitCode.INVALID_DEPLOY_CONFIG)
    except FileNotFoundError:
        logging.exception("Could not find config/config.json.")
        sys.exit(ExitCode.NO_DEPLOY_JSON)

    return config



import subprocess
import logging
import os


class Bash:
    @staticmethod
    def run_command(args, env=None) -> subprocess.CompletedProcess:
        logging.debug(f"Running the following command: {args}")

        new_env = os.environ

        if env:
            new_env.update(env)

        return subprocess.run(
            args=args, env=new_env, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT
        )



import click
import sys
import logging
from rli.cli import CONTEXT_SETTINGS
from rli.git import RLIGit
from rli.deploy import RLIDeploy
from rli.config import DeployConfig, get_rli_config_or_exit, get_deploy_config_or_exit
from rli.constants import ExitCode


@click.command(
    "deploy",
    context_settings=CONTEXT_SETTINGS,
    help="Runs a deploy. You must be in the directory that has the "
    "deploy/deploy.json file. This command will attempt to checkout the commit "
    "inputted unless you use the '--skip-checkout' flag. When running a docker "
    "deploy, the image with the tag corresponding to the commit you pass in is "
    "pulled. The image is then tagged with 'latest' so it will be then image "
    "ran by docker-compose. If the commit specified is 'master', it pulls the "
    "image with the 'latest' tag.",
)
@click.option(
    "--commit",
    default="master",
    help="The git commit you want to deploy. 'master' is the default.",
)
@click.option(
    "--skip-checkout",
    is_flag=True,
    help="If this flag is used, checkout will be skipped",
)
@click.pass_context
def cli(ctx, commit, skip_checkout):
    logging.info(
        f"Running deploy for commit {commit}.{' Skipping checkout.' if skip_checkout else ''}"
    )
    rli_config = get_rli_config_or_exit()
    rli_deploy_config = get_deploy_config_or_exit()

    if not skip_checkout:
        checkout = RLIGit.checkout(commit)

        if checkout != 0:
            logging.error(f"Could not checkout commit: {commit}")
            sys.exit(ExitCode.GIT_ERROR)

    deploy = RLIDeploy(rli_config, rli_deploy_config).run_deploy(commit)

    if deploy is None or deploy != 0:
        logging.error("Deploy did not finish successfully.")
        sys.exit(ExitCode.DEPLOY_FAILED)
    else:
        logging.info("Deploy finished successfully.")



class InvalidRLIConfiguration(Exception):
    def __init__(self, *args):
        self.message = args[0] if args else None

    def __str__(self):
        if self.message:
            return f"InvalidRLIConfiguration has been raised: {self.message}"
        else:
            return "InvalidRLIConfiguration has been raised."


class InvalidDeployConfiguration(Exception):
    def __init__(self, *args):
        self.message = args[0] if args else None

    def __str__(self):
        if self.message:
            return f"InvalidDeployConfiguration has been raised: {self.message}"
        else:
            return "InvalidDeployConfiguration has been raised."


class RLIDockerException(Exception):
    def __init__(self, *args):
        self.message = args[0] if args else None

    def __str__(self):
        if self.message:
            return f"RLIDockerException has been raised: {self.message}"
        else:
            return "RLIDockerException has been raised."



import logging
import os
from rli.exceptions import RLIDockerException
from rli.utils.bash import Bash


class RLIDocker:
    def __init__(self, username, password, registry):
        self.username = username
        self.password = password
        self.registry = registry if registry[-1] == "/" else registry + "/"
        self.login()

    def login(self):
        if (
            Bash.run_command(
                [
                    "echo",
                    f'"{self.password}"',
                    "|",
                    "docker",
                    "login",
                    "-u",
                    self.username,
                    "--password-stdin",
                    self.registry,
                ]
            ).returncode
            != 0
        ):
            raise RLIDockerException("Could not log into the provided Docker registry.")

    def pull(self, image):
        """
        Pulls a image from the registry passed in to the constructor. E.g.
        image='ubuntu', docker pull some.registry.com/ubuntu
        :param image: The name of the image
        :return: The full image name if successful, otherwise None
        """
        if (
            Bash.run_command(["docker", "pull", f"{self.registry}{image}"]).returncode
            != 0
        ):
            return None
        else:
            return f"{self.registry}{image}"

    def tag(self, current_tag, new_tag):
        """
        Tags a docker images with the new tag.
        :param current_tag: The current tag
        :param new_tag: The new tag
        :return: The new tag if the command is successful, otherwise None
        """
        if Bash.run_command(["docker", "tag", current_tag, new_tag]).returncode != 0:
            return None
        else:
            return new_tag

    def compose_up(self, compose_file, secrets):
        """
        Runs docker-compose up for the given docker-compose file. The given
        secrets tuple is passed in as well.

        :param compose_file: The docker-compose file
        :param secrets: A dict of the secrets
        :return: The exit code of the cli command
        """
        return Bash.run_command(
            args=["docker-compose", "-f", compose_file, "up", "-d"], env=secrets
        ).returncode

    def run_image(self, image, secrets):
        """
        Runs a the Docker image you pass in along with the secrets as environment variables for Docker image.
        :param image: The name of the image
        :param secrets: The secrets to pass in
        :return: The exit code of the command
        """
        args = ["docker", "run", "-d"]

        for key, value in secrets.items():
            args.append("-e")
            args.append(f"{key}={value}")

        args.append(image)

        return Bash.run_command(args=args, env=secrets).returncode



from github import Github, GithubException
import logging


class RLIGithub:
    def __init__(self, config):
        self.github = (
            Github(config.login, config.password)
            if config.password
            else Github(config.login)
        )
        self.config = config

    def create_repo(self, repo_name, repo_description="", private="false"):
        logging.debug(f"Creating repo '{repo_name}'.")
        private = private == "true"

        try:
            return self.github.get_user().create_repo(
                repo_name,
                description=repo_description,
                private=private,
                auto_init=True,
            )
        except GithubException as e:
            if e.status == 422:
                logging.error("Repository name is taken.")
            else:
                logging.error("There was an exception when creating your repository.")



import click
import logging
import sys
from rli.cli import CONTEXT_SETTINGS
from rli.constants import ExitCode


@click.command(
    "smoke",
    context_settings=CONTEXT_SETTINGS,
    help="Command to run to make sure RLI is installed.",
)
@click.pass_context
def cli(ctx):
    logging.info("RLI is working properly.")
    sys.exit(ExitCode.OK)



from rli.utils.bash import Bash
import logging


class RLIGit:
    @staticmethod
    def checkout(commit_or_branch):
        logging.debug(f"Checking out: {commit_or_branch}")
        return Bash.run_command(["git", "checkout", commit_or_branch]).returncode



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rli', 'rli.commands', 'rli.utils']

package_data = \
{'': ['*']}

install_requires = \
['PyGithub>=1.47,<2.0', 'click>=7.1.1,<8.0.0']

entry_points = \
{'console_scripts': ['rli = rli.cli:cli']}

setup_kwargs = {
    'name': 'rli',
    'version': '0.0.2',
    'description': 'My CLI tool to automate common tasks.',
    'long_description': None,
    'author': 'luke.shay',
    'author_email': 'luke.shay@vertexvis.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from rli.docker import RLIDocker
from rli.config import DockerDeployConfig, DockerConfig, RLIConfig, DeployConfig
import logging


class RLIDeploy:
    def __init__(self, rli_config, deploy_config):
        self._rli_config = rli_config
        self._deploy_config = deploy_config

        self._rli_docker = None

    def run_deploy(self, commit):
        if self.docker_deploy_config:
            logging.debug(f"Running Docker deploy for commit: {commit}")
            commit = "latest" if commit == "master" else commit

            image = self.rli_docker.pull(f"{self.docker_deploy_config.image}:{commit}")

            if not image:
                logging.error("There was an error pulling the docker image.")
                return

            tag = self.rli_docker.tag(
                image, f"{self.docker_deploy_config.image}:latest"
            )

            if not tag:
                logging.error("There was an error tagging the docker image.")
                return

            if self.docker_deploy_config.compose_file:
                return self.rli_docker.compose_up(
                    self.docker_deploy_config.compose_file, self.rli_config.rli_secrets,
                )
            else:
                return self.rli_docker.run_image(
                    f"{self.docker_deploy_config.image}:latest",
                    self.rli_config.rli_secrets,
                )

    # These methods are used for improved intellisense
    @property
    def docker_deploy_config(self) -> DockerDeployConfig:
        return self.deploy_config.docker_deploy_config

    @property
    def docker_config(self) -> DockerConfig:
        return self.rli_config.docker_config

    @property
    def rli_config(self) -> RLIConfig:
        return self._rli_config

    @property
    def deploy_config(self) -> DeployConfig:
        return self._deploy_config

    @property
    def rli_docker(self) -> RLIDocker:
        if not self._rli_docker:
            self._rli_docker = RLIDocker(
                self.docker_config.login,
                self.docker_config.password,
                self.docker_config.registry,
            )

        return self._rli_docker
